@@ -3,7 +3,7 @@
   Copyright (C) 2016 Nikolaus Rath <Nikolaus@rath.org>
 
   This program can be distributed under the terms of the GNU GPLv2.
-  See the file COPYING.
+  See the file GPL2.txt.
 */
 
 /** @file
