@@ -2,8 +2,8 @@
   FUSE: Filesystem in Userspace
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
-  This program can be distributed under the terms of the GNU GPL.
-  See the file COPYING.
+  This program can be distributed under the terms of the GNU GPLv2.
+  See the file GPL2.txt.
 */
 
 /** @file
@@ -22,13 +22,12 @@
  */
 
 
-#define FUSE_USE_VERSION 30
-
-#include <config.h>
+#define FUSE_USE_VERSION 31
 
 #include <fuse.h>
 #include <fuse_lowlevel.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <time.h>
@@ -105,7 +104,7 @@ static int null_write(const char *path, const char *buf, size_t size,
 	return size;
 }
 
-static struct fuse_operations null_oper = {
+static const struct fuse_operations null_oper = {
 	.getattr	= null_getattr,
 	.truncate	= null_truncate,
 	.open		= null_open,
@@ -121,6 +120,7 @@ int main(int argc, char *argv[])
 
 	if (fuse_parse_cmdline(&args, &opts) != 0)
 		return 1;
+	fuse_opt_free_args(&args);
 
 	if (!opts.mountpoint) {
 		fprintf(stderr, "missing mountpoint parameter\n");
@@ -130,8 +130,10 @@ int main(int argc, char *argv[])
 	if (stat(opts.mountpoint, &stbuf) == -1) {
 		fprintf(stderr ,"failed to access mountpoint %s: %s\n",
 			opts.mountpoint, strerror(errno));
+		free(opts.mountpoint);
 		return 1;
 	}
+	free(opts.mountpoint);
 	if (!S_ISREG(stbuf.st_mode)) {
 		fprintf(stderr, "mountpoint is not a regular file\n");
 		return 1;
