@@ -6,7 +6,7 @@
   fuse_bufvec`.
 
   This program can be distributed under the terms of the GNU LGPLv2.
-  See the file COPYING.LIB
+  See the file LGPL2.txt
 */
 
 #define _GNU_SOURCE
