@@ -1,3 +1,12 @@
+
+#include "fuse_config.h"
+
+#ifdef HAVE_PTHREAD_SETNAME_NP
+#define _GNU_SOURCE
+#include <pthread.h>
+#endif
+
+#include <errno.h>
 #include <stdlib.h>
 #include <errno.h>
 
@@ -33,3 +42,12 @@ int libfuse_strtol(const char *str, long *res)
 	return 0;
 }
 
+void fuse_set_thread_name(const char *name)
+{
+#ifdef HAVE_PTHREAD_SETNAME_NP
+	pthread_setname_np(pthread_self(), name);
+#else
+	(void)name;
+#endif
+}
+
