@@ -12,6 +12,7 @@
 struct fuse_conn_info;
 
 int libfuse_strtol(const char *str, long *res);
+void fuse_set_thread_name(const char *name);
 
 /**
  * Return the low bits of a number
@@ -30,7 +31,7 @@ static inline uint64_t fuse_higher_32_bits(uint64_t nr)
 }
 
 #ifndef FUSE_VAR_UNUSED
-#define FUSE_VAR_UNUSED(var) (__attribute__((unused)) var)
+#define FUSE_VAR_UNUSED __attribute__((__unused__))
 #endif
 
 #define container_of(ptr, type, member)                      \
@@ -39,4 +40,10 @@ static inline uint64_t fuse_higher_32_bits(uint64_t nr)
 		((type *)(__mptr - offsetof(type, member))); \
 	})
 
+#if __has_attribute(__fallthrough__)
+#define fallthrough __attribute__((__fallthrough__))
+#else
+#define fallthrough do {} while (0)
 #endif
+
+#endif /* FUSE_UTIL_H_ */
