@@ -3,7 +3,7 @@
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
   This program can be distributed under the terms of the GNU GPLv2.
-  See the file COPYING.
+  See the file GPL2.txt.
 */
 /* This program does the mounting and unmounting of FUSE filesystems */
 
@@ -36,11 +36,18 @@
 #include <stdbool.h>
 #include <sys/vfs.h>
 
-#ifdef HAVE_CLOSE_RANGE
+#if defined HAVE_CLOSE_RANGE && defined linux
 #include <linux/close_range.h>
 #endif
 
+#if defined HAVE_LISTMOUNT
+#include <linux/mount.h>
+#include <syscall.h>
+#include <stdint.h>
+#endif
+
 #define FUSE_COMMFD_ENV		"_FUSE_COMMFD"
+#define FUSE_KERN_DEVICE_ENV	"FUSE_KERN_DEVICE"
 
 #define FUSE_DEV "/dev/fuse"
 
@@ -565,7 +572,7 @@ static int unmount_fuse(const char *mnt, int quiet, int lazy)
 	return res;
 }
 
-static int count_fuse_fs(void)
+static int count_fuse_fs_mtab(void)
 {
 	struct mntent *entp;
 	int count = 0;
@@ -585,6 +592,72 @@ static int count_fuse_fs(void)
 	return count;
 }
 
+#ifdef HAVE_LISTMOUNT
+static int count_fuse_fs_ls_mnt(void)
+{
+	#define SMBUF_SIZE 1024
+	#define MNT_ID_LEN 128
+
+	int fuse_count = 0;
+	int n_mounts = 0;
+	int ret = 0;
+	uint64_t mnt_ids[MNT_ID_LEN];
+	unsigned char smbuf[SMBUF_SIZE];
+	struct mnt_id_req req = {
+		.size = sizeof(struct mnt_id_req),
+	};
+	struct statmount *sm;
+
+	for (;;) {
+		req.mnt_id = LSMT_ROOT;
+
+		n_mounts = syscall(SYS_listmount, &req, &mnt_ids, MNT_ID_LEN, 0);
+		if (n_mounts == -1) {
+			if (errno != ENOSYS) {
+				fprintf(stderr, "%s: failed to list mounts: %s\n", progname,
+					strerror(errno));
+			}
+			return -1;
+		}
+
+		for (int i = 0; i < n_mounts; i++) {
+			req.mnt_id = mnt_ids[i];
+			req.param = STATMOUNT_FS_TYPE;
+			ret = syscall(SYS_statmount, &req, &smbuf, SMBUF_SIZE, 0);
+			if (ret) {
+				if (errno == ENOENT)
+					continue;
+
+				fprintf(stderr, "%s: failed to stat mount %lld: %s\n", progname,
+					req.mnt_id, strerror(errno));
+				return -1;
+			}
+
+			sm = (struct statmount *)smbuf;
+			if (sm->mask & STATMOUNT_FS_TYPE &&
+			    strcmp(&sm->str[sm->fs_type], "fuse") == 0)
+				fuse_count++;
+		}
+
+		if (n_mounts < MNT_ID_LEN)
+			break;
+		req.param = mnt_ids[MNT_ID_LEN - 1];
+	}
+	return fuse_count;
+}
+
+static int count_fuse_fs(void)
+{
+	int count = count_fuse_fs_ls_mnt();
+
+	return count >= 0 ? count : count_fuse_fs_mtab();
+}
+#else
+static int count_fuse_fs(void)
+{
+	return count_fuse_fs_mtab();
+}
+#endif
 
 #else /* IGNORE_MTAB */
 static int count_fuse_fs(void)
@@ -1146,6 +1219,7 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *mountpoint_fd)
 		0x73717368 /* SQUASHFS_MAGIC */,
 		0x01021994 /* TMPFS_MAGIC */,
 		0x24051905 /* UBIFS_SUPER_MAGIC */,
+		0x18031977 /* WEKAFS_SUPER_MAGIC */,
 #if __SIZEOF_LONG__ > 4
 		0x736675005346544e /* UFSD */,
 #endif
@@ -1163,56 +1237,30 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *mountpoint_fd)
 	return -1;
 }
 
-static int try_open(const char *dev, char **devp, int silent)
-{
-	int fd = open(dev, O_RDWR);
-	if (fd != -1) {
-		*devp = strdup(dev);
-		if (*devp == NULL) {
-			fprintf(stderr, "%s: failed to allocate memory\n",
-				progname);
-			close(fd);
-			fd = -1;
-		}
-	} else if (errno == ENODEV ||
-		   errno == ENOENT)/* check for ENOENT too, for the udev case */
-		return -2;
-	else if (!silent) {
-		fprintf(stderr, "%s: failed to open %s: %s\n", progname, dev,
-			strerror(errno));
-	}
-	return fd;
-}
-
-static int try_open_fuse_device(char **devp)
+static int open_fuse_device(const char *dev)
 {
 	int fd;
 
 	drop_privs();
-	fd = try_open(FUSE_DEV, devp, 0);
+	fd = open(dev, O_RDWR);
+	if (fd == -1) {
+		if (errno == ENODEV || errno == ENOENT)/* check for ENOENT too, for the udev case */
+			fprintf(stderr,
+				"%s: fuse device %s not found. Kernel module not loaded?\n",
+				progname, dev);
+		else
+			fprintf(stderr,
+				"%s: failed to open %s: %s\n", progname, dev, strerror(errno));
+	}
 	restore_privs();
 	return fd;
 }
 
-static int open_fuse_device(char **devp)
-{
-	int fd = try_open_fuse_device(devp);
-	if (fd >= -1)
-		return fd;
-
-	fprintf(stderr,
-		"%s: fuse device not found, try 'modprobe fuse' first\n",
-		progname);
-
-	return -1;
-}
-
-
 static int mount_fuse(const char *mnt, const char *opts, const char **type)
 {
 	int res;
 	int fd;
-	char *dev;
+	const char *dev = getenv(FUSE_KERN_DEVICE_ENV) ?: FUSE_DEV;
 	struct stat stbuf;
 	char *source = NULL;
 	char *mnt_opts = NULL;
@@ -1221,7 +1269,7 @@ static int mount_fuse(const char *mnt, const char *opts, const char **type)
 	char *do_mount_opts = NULL;
 	char *x_opts = NULL;
 
-	fd = open_fuse_device(&dev);
+	fd = open_fuse_device(dev);
 	if (fd == -1)
 		return -1;
 
@@ -1292,7 +1340,6 @@ static int mount_fuse(const char *mnt, const char *opts, const char **type)
 out_free:
 	free(source);
 	free(mnt_opts);
-	free(dev);
 	free(x_opts);
 	free(do_mount_opts);
 
